﻿using BPA.Message.Enum;
using BPASmartClient.Helper;
using BPASmartClient.Peripheral;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace BPASmartClient.Device
{
    /// <summary>
    /// 设备基类
    /// </summary>
    public abstract class BaseDevice : IDevice
    {
        public BaseDevice()
        {
        }

        /// <summary>
        /// 设备ID
        /// </summary>
        public int DeviceId { get; set; }
        /// <summary>
        /// 设备所有状态
        /// </summary>
        public DeviceStatus Status { get; set; } = new DeviceStatus();
        /// <summary>
        /// 设备名称
        /// </summary>
        public string Name { get; set; }
        /// <summary>
        /// 设备类型
        /// </summary>
        public abstract DeviceClientType DeviceType { get; }
        /// <summary>
        /// 是否忙碌
        /// </summary>
        public bool IsBusy { get; protected set; }
        /// <summary>
        /// 是否健康
        /// </summary>
        public bool IsHealth { get; protected set; }

        //外设状态
        protected ConcurrentDictionary<string, object> peripheralStatus = new ConcurrentDictionary<string, object>();
        private List<IPeripheral> peripherals;

        public void Initliaze()
        {
        }

        public void Initliaze(List<IPeripheral> peripherals)
        {
            peripherals.ForEach(p =>
            {
                p.DeviceId = this.DeviceId;
                p.Init();
            });
            this.peripherals = peripherals;
        }

        public virtual void StartMain()
        {
            ThreadManage.GetInstance().StartLong(new Action(() =>
            {
                foreach (var peripheral in peripherals)
                {
                    foreach (var key in peripheral.GetAllStatus().Keys)
                    {
                        peripheralStatus[key] = peripheral.GetAllStatus()[key];
                    }
                }
                Thread.Sleep(100);
            }), "GetAllStatus");
            DoMain();
        }

        public abstract void DoMain();
        public abstract void Stop();
    }
}
